program QuickSorter;
{	Sample file for KPascal.
	This file is intended to introduce the functionality of KPascal V0.1	
	
	At first, things that are bad in KPascal:
	1) It is not yet tested thoroughly and hell knows whether it
	   will compile everything correctly
	2) Errors and warnings that it reports are to be improved
	   Sometimes instead of reporting an error the compiler will
	   generate an exception.
	3) No range checking for arrays
	4) Efficiency is not an issue - there is no such thing here
	5) Those things are still undone and may crash the compiler:
		A = ^A
	  and
		PType = ^TType;
		TType = ...
	6) Too few built-in functions, few I/O capabilities
	7) Not much of available memory
	8) Support for Pascal-Strings only, no String-handling functions.
	9) Editor is not perfect (that seems to be mostly a problem of RichEdit,
	   so it's not my fault)
	10) Interpreter is far not perfect.
	11) Editor crashes if the window of the program being interpreted is
	   closed forcefully.
	12) Doesn't compile to native-code. (However, this will (hopefully) 
	    solved in the future versions)
	    (Note that now it produces (for debugging purposes) a .kpc
	    file with the corresponding KPCode program)
	13) etc.. you are welcome to continue this list and mail it to me	

	Copyright  2002, Konstantin Tretyakov
}

{	Features supported by KPascal:	}

{ 1)	Basic types are: Integer (32 bit), Real, String, 
	                 Boolean (8 bit), Pointer (32 bit)
  2)	Allows to define custom record/pointer/array (also multidimensional) types	 }
type
	TArrayItem = record Key: Integer; Data: String end;
{ 3)  Constants of basic types:	}
const 
	ARRAY_SIZE = 20;
type
	TItemArray = array[0..ARRAY_SIZE-1] of TArrayItem;

{ 4) Functions:	}
function InitArray: TItemArray;
var i: Integer;
begin
{ 5) "for" loops:	}
	for i := 0 to ARRAY_SIZE-1 do begin
{ 6) "Result" variable, "IntToStr", "FloatToStr"	}
		InitArray[i].Key := ARRAY_SIZE-1-i;
		Result[i].Data := 'String number ' + IntToStr(ARRAY_SIZE - 1 - i);
	end;
end;

{ 7) "forward" directive for procedures/functions	}
procedure QuickSort(var A: TItemArray); forward;
{ 8) Procedures, nested procedures, "var" parameters	}
procedure QuickSort(var A: TItemArray);
	procedure QS(var A: TItemArray; Lo, Hi: Integer);
	var iLo, iHi: Integer; M: Integer; 
		T:TArrayItem;
	begin
		iLo := Lo;
		iHi := Hi;
		M := A[(iLo + iHi) div 2].Key;
{ 9) "repeat" and "while" statements	}
		repeat
			while A[iLo].Key < M do Inc(iLo);
			while A[iHi].Key > M do Dec(iHi);
			if iLo <= iHi then begin
				T := A[iHi];
				A[iHi] := A[iLo];
				A[iLo] := T;
				iLo := iLo + 1;
				iHi := iHi - 1;
			end;
		until	iHi < iLo;
		if iHi > Lo then QS(A, Lo, iHi);
		if iLo < Hi then QS(A, iLo, Hi);	
	end;
begin
	QS(A, 0, ARRAY_SIZE-1);
end;

var i: Integer; A:TItemArray; s: String;
{10) Labels and "goto" statements	}
label Start;
begin
{11) ReadLn, Write, WriteLn
     (However all of them only accept one argument,
      For ReadLn it may be a String/Integer/Real,
      For WriteLn - only a String	}
	WriteLn('Hello, you will now see how QuickSort works');
Start: 
{12) Though Char type is not supported, you may use the #n notation}
	WriteLn('Initializing'#32' array...');
	A := InitArray;
	WriteLn('Here is an unsorted array:');
	for i := 0 to ARRAY_SIZE - 1 do WriteLn(A[i].Data);
	Write('Press Enter to continue...');
	ReadLn;
	WriteLn('Sorting...');
	QuickSort(A);
	WriteLn('Here is this array sorted: ');
	for i := 0 to ARRAY_SIZE - 1 do WriteLn(A[i].Data);
	Write('Cool, isn''t it? Want to try again? [Y/N]:');
	ReadLn(s);
{13) "Exit", "Halt", "Break" and "Continue" statements, SizeOf(	}
	if (s = 'N') or (s = 'n') then Exit
		else goto Start;
end.
