program Maze2;
{	Sample file nr. 3 for KPascal.

	Kruskal algorithm with Galler-Fisher class merging.
	Maze will be more "interesting" than that made with a DFS

	Copyright  2002, Konstantin Tretyakov
}

type
	PPtr = ^PPtr;
	TEdge = record X,Y: Integer; Used: Boolean; end;
const
	MAZE_ROWS = 23;	{Must be odd}
	MAZE_COLS = 79;	{Must be odd}
	VERTICES_IN_ROW = MAZE_COLS div 2;
	VERTICES_IN_COL = MAZE_ROWS div 2;
	VERTEX_COUNT = VERTICES_IN_ROW * VERTICES_IN_COL;
	EDGE_COUNT = 2 * VERTEX_COUNT - VERTICES_IN_ROW - VERTICES_IN_COL;

var 
	A: array[1..MAZE_ROWS, 1..MAZE_COLS] of Boolean;
	C: array[1..MAZE_ROWS, 1..MAZE_COLS] of PPtr;
	Edges: array[0..EDGE_COUNT-1] of TEdge;

function FindRepresentative(A: PPtr): PPtr;
begin
	if A^ = PPtr(nil) then Result := A
	else begin
		Result := FindRepresentative(A^);
		A^ := Result;
	end;
end;

function SameClass(A,B: PPtr): Boolean;
begin
	Result := FindRepresentative(A) = FindRepresentative(B);
end;

procedure MergeClasses(A,B: PPtr);
var U, V: PPtr;
begin
	U := FindRepresentative(A);
	V := FindRepresentative(B);
	if U <> V then U^ := V;
end;


procedure InitMaze; 
var i, j, N: Integer;
begin
	N := -1;
	for i := 1 to MAZE_ROWS do
		for j := 1 to MAZE_COLS do begin
			A[i,j] := Odd(i) or Odd(j);
			C[i,j] := PPtr(nil);
			if (Odd(i) xor Odd(j)) then begin
				if (j=1) or (j=MAZE_COLS) then Continue;
				if (i=1) or (i=MAZE_ROWS) then Continue;
				Inc(N);
				Edges[N].X := i;
				Edges[N].Y := j;
				Edges[N].Used := False;
			end;
		end;
end;

procedure TakeAwayRandomEdge;
var i, X, Y: Integer;
label Seek;
begin
	i := Random(EDGE_COUNT);
Seek:
	while Edges[i].Used do i := (i + 1) mod EDGE_COUNT;
	X := Edges[i].X;
	Y := Edges[i].Y;
	Edges[i].Used := True;
	if Odd(X) then begin {Horizontal edge}
		if SameClass(@C[X-1,Y], @C[X+1,Y]) then goto Seek
		else begin
			MergeClasses(@C[X-1,Y], @C[X+1,Y]);
			A[X,Y] := False;
		end;
	end else begin {Vertical edge}
		if SameClass(@C[X,Y-1], @C[X,Y+1]) then goto Seek
		else begin
			MergeClasses(@C[X,Y-1], @C[X,Y+1]);
			A[X,Y] := False;
		end;
	end;
end;

procedure PrintMaze;
var i, j: Integer; S: String;
begin
	for i := 1 to MAZE_ROWS do begin
		S := '';
		for j := 1 to MAZE_COLS do if A[i,j] = True then
			S := S + #219 {white square, use #2 for smileys}
		else S := S + ' ';
		WriteLn(S);
	end;
	WriteLn('');
end;

label Start;
var i: Integer; S: String;
begin
Start:
	WriteLn('Wait a sec...');
	InitMaze;
{	Take away randomly V-1 edges	}
	for i := 1 to VERTEX_COUNT-1 do TakeAwayRandomEdge;
	PrintMaze;
	Write('Strike ENTER for more or enter N to quit:');
	ReadLn(S);
	if S = '' then goto Start;
end.


